@@ -49,6 +49,9 @@
 #define MAV_COLOUR_GREEN -13
 #define MAV_COLOUR_BLUE  -14
 
+#define MAV_LIGHT_RELATIVE 0
+#define MAV_LIGHT_ABSOLUTE 1
+
 #define MAV_SILENT 0
 #define MAV_VERBOSE 1
 
@@ -64,6 +67,15 @@
 #define MAV_MATRIX_XCOMP 0][3
 #define MAV_MATRIX_YCOMP 1][3
 #define MAV_MATRIX_ZCOMP 2][3
+#define MAV_MATRIX_XAXIS_X 0][0
+#define MAV_MATRIX_XAXIS_Y 1][0
+#define MAV_MATRIX_XAXIS_Z 2][0
+#define MAV_MATRIX_YAXIS_X 0][1
+#define MAV_MATRIX_YAXIS_Y 1][1
+#define MAV_MATRIX_YAXIS_Z 2][1
+#define MAV_MATRIX_ZAXIS_X 0][2
+#define MAV_MATRIX_ZAXIS_Y 1][2
+#define MAV_MATRIX_ZAXIS_Z 2][2
 
 #define MAV_REDEFINE_WARN 1
 #define MAV_REDEFINE_NOWARN 2
@@ -196,6 +208,7 @@ typedef struct {
   float diffuse[4];
   float specular[4];
   MAV_vector pos;
+  int positioning;
 } MAV_light; 
 
 typedef struct {
@@ -423,6 +436,7 @@ void mav_paletteTextureFree(MAV_palette *p, int index);
 void mav_paletteLightSet(MAV_palette *p, int index, float ar, float ag, float ab, float aa, 
 		  float dr, float dg, float db, float da, float sr, float sg, float sb, float sa);
 void mav_paletteLightPos(MAV_palette *p, int index, MAV_vector pos);
+void mav_paletteLightPositioning(MAV_palette *p, int index, int pos);
 void mav_paletteLightingModelSet(MAV_palette *p, float ar, float ag, float ab, float aa, int local);
 void mav_paletteFontSet(MAV_palette *p, int index, char *s);
 
@@ -474,6 +488,12 @@ MAV_vector mav_matrixXYZGet(MAV_matrix m);
 void mav_matrixRPYGet(MAV_matrix m, float *r, float *p, float *y);
 MAV_matrix mav_matrixScaleSet(MAV_matrix m, float sc);
 float mav_matrixScaleGet(MAV_matrix m);
+MAV_vector mav_matrixXAxisGet(MAV_matrix m);
+MAV_vector mav_matrixYAxisGet(MAV_matrix m);
+MAV_vector mav_matrixZAxisGet(MAV_matrix m);
+MAV_matrix mav_matrixXAxisSet(MAV_matrix m, MAV_vector v);
+MAV_matrix mav_matrixYAxisSet(MAV_matrix m, MAV_vector v);
+MAV_matrix mav_matrixZAxisSet(MAV_matrix m, MAV_vector v);
 MAV_matrix mav_matrixMult(MAV_matrix m1, MAV_matrix m2);
 MAV_matrix mav_matrixInverse(MAV_matrix m);
 MAV_matrix mav_matrixQuaternionConvert(MAV_quaternion q);
@@ -563,12 +583,16 @@ int mav_SMSObjectRmv(MAV_SMS *s, MAV_object *o);
 /* Global variables */
 
 extern const MAV_vector MAV_NULL_VECTOR;
+extern const MAV_vector MAV_X_VECTOR;
+extern const MAV_vector MAV_Y_VECTOR;
+extern const MAV_vector MAV_Z_VECTOR;
 extern const MAV_matrix MAV_ID_MATRIX;
 extern const MAV_quaternion MAV_ID_QUATERNION;
 
 extern MAV_list *mav_win_list;
 extern MAV_list *mav_sms_list;
 extern MAV_list *mav_module_list;
+extern MAV_list *mav_palette_list;
 
 extern MAV_window *mav_win_all;
 extern MAV_window *mav_win_orig;
