@@ -83,7 +83,25 @@ int mavlib_TRKey(MAV_object *o, MAV_keyboardEvent *ke)
       rv= -862;
       mavlib_TROS=0;
     }
-   if (ke->key==13) rv=-862;
+   if (ke->key==13) rv= -862;
+  }
+  
+  return rv;
+}
+
+int mavlib_TRKey2(MAV_object *o, MAV_keyboardEvent *ke)
+{
+  int rv=-1;
+
+  if (ke->movement==MAV_PRESSED) {
+    if (ke->key=='a') rv= -862;
+    if (ke->key==27) rv= -862;
+    if (ke->key==13) rv= -862;
+
+    if (ke->key=='o') {
+      rv= -862;
+      mavlib_TROS= 0;
+    }
   }
   
   return rv;
@@ -91,7 +109,7 @@ int mavlib_TRKey(MAV_object *o, MAV_keyboardEvent *ke)
 
 
 
-void mavlib_TRDump(MAV_window *w, int os, int dl)
+void mavlib_TRDump(MAV_window *w, int os, int dl, int aa)
 {
 #ifdef MAV_TR
   TRcontext *tr;
@@ -113,7 +131,14 @@ void mavlib_TRDump(MAV_window *w, int os, int dl)
   tr = trNew();
   trTileSize(tr, w->width, w->height, 0);
   tile = mav_malloc(w->width * w->height * 3 * sizeof(GLubyte));
-  img = mav_malloc(w->width * w->height * 3 * sizeof(GLubyte));
+  if (aa)
+  {
+    img = mav_malloc(w->width * w->height * 3 * sizeof(GLubyte));
+  }
+  else
+  {
+    img = mav_malloc(w->width * os * w->height * os * 3 * sizeof(GLubyte));
+  }
   trTileBuffer(tr, GL_RGB, GL_UNSIGNED_BYTE, tile);
   trImageSize(tr, w->width*os, w->height*os);
   trPerspective(tr, w->fov, w->aspect, w->ncp, w->fcp);
@@ -130,17 +155,31 @@ void mavlib_TRDump(MAV_window *w, int os, int dl)
     mav_gfxWindowBuffersSwap();
     more = trEndTile(tr);
 
-    /* Scale tile and copy into final image */
-    for (sy=0, dy=r*w->height/os; sy<w->height; sy+=os, dy++) {
-      for (sx=0, dx=c*w->width/os; sx<w->width; sx+=os, dx++) {
-	for (k=0; k<3; k++) {	    
-	  v=0;
-	  for (i=0; i<os; i++) {
-	    for (j=0; j<os; j++) {
-	      v+= tile[((sy+j)*w->width*3)+((sx+i)*3)+k];
+    if (aa) 
+    {
+      /* Scale tile and copy into final image */
+      for (sy=0, dy=r*w->height/os; sy<w->height; sy+=os, dy++) {
+	for (sx=0, dx=c*w->width/os; sx<w->width; sx+=os, dx++) {
+	  for (k=0; k<3; k++) {	    
+	    v=0;
+	    for (i=0; i<os; i++) {
+	      for (j=0; j<os; j++) {
+		v+= tile[((sy+j)*w->width*3)+((sx+i)*3)+k];
+	      }
 	    }
+	    img[(dy*w->width*3)+(dx*3)+k]= v/(os*os);
+	  }
+	}
+      }
+    }
+    else
+    {
+      /* Copy into final image */
+      for (sy=0, dy=r*w->height; sy<w->height; sy++, dy++) {
+	for (sx=0, dx=c*w->width; sx<w->width; sx++, dx++) {
+	  for (k=0; k<3; k++) {
+	    img[(dy*w->width*os*3)+(dx*3)+k]= tile[(sy*w->width*3)+(sx*3)+k];
 	  }
-	  img[(dy*w->width*3)+(dx*3)+k]= v/(os*os);
 	}
       }
     }
@@ -162,9 +201,17 @@ void mavlib_TRDump(MAV_window *w, int os, int dl)
 
     return;
   }
+
   fprintf(f,"P6\n");
-  fprintf(f,"# ppm-file created by Maverik\n");
-  fprintf(f,"%i %i\n", w->width, w->height);
+  fprintf(f,"# PPM-file created by Maverik\n");
+  if (aa)
+  {
+    fprintf(f,"%i %i\n", w->width, w->height);
+  }
+  else
+  {
+    fprintf(f,"%i %i\n", w->width*os, w->height*os);
+  }
   fprintf(f,"255\n");
   fclose(f);
   f = fopen(filename, "ab");  /* now append binary data */
@@ -182,11 +229,23 @@ void mavlib_TRDump(MAV_window *w, int os, int dl)
     return;
   }
   
-  for (i=0;i<w->height;i++) {
-    /* Remember, OpenGL images are bottom to top.  Have to reverse. */
-    rowPtr = img + (w->height-1-i) * w->width*3;
-    fwrite(rowPtr, 1, w->width*3, f);
+  if (aa) 
+  {
+    for (i=0; i<w->height; i++) {
+      /* Remember, OpenGL images are bottom to top.  Have to reverse. */
+      rowPtr = img + (w->height-1-i) * w->width*3;
+      fwrite(rowPtr, 1, w->width*3, f);
+    }
   }
+  else
+  {
+    for (i=0; i<w->height*os; i++) {
+      /* Remember, OpenGL images are bottom to top.  Have to reverse. */
+      rowPtr = img + (w->height*os-1-i) * w->width*os*3;
+      fwrite(rowPtr, 1, w->width*os*3, f);
+    }
+  }
+
   fclose(f);
 
   if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "written %s\n", filename);
@@ -212,6 +271,7 @@ void mavlib_TRfn3(void)
   MAV_timer tim;
   int os=4, rv;
   float lw;
+  int aa=0;
 
   if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "done.\n");   
 
@@ -247,13 +307,13 @@ void mavlib_TRfn3(void)
   /* Display image and prompt for number of oversamples */
   mav_windowSet(mav_win_left);
   mav_gfxListExec(DL);
-  mav_stringDisplay(mav_win_left, "Got this image. Number of oversamples? (2, 4, or 8)\n", MAV_COLOUR_BLACK, 0, -0.9, 0.9);
+  mav_stringDisplay(mav_win_left, "Got this image. Number of oversamples? (2, 4, or 8)", MAV_COLOUR_BLACK, 0, -0.9, 0.9);
   mav_gfxWindowBuffersSwap();
 
   if (mav_opt_stereo) {
     mav_windowSet(mav_win_right);
     mav_gfxListExec(DR);
-    mav_stringDisplay(mav_win_right, "Got this image. Number of oversamples? (2, 4, or 8)\n", MAV_COLOUR_BLACK, 0, -0.9, 0.9);
+    mav_stringDisplay(mav_win_right, "Got this image. Number of oversamples? (2, 4, or 8)", MAV_COLOUR_BLACK, 0, -0.9, 0.9);
     mav_gfxWindowBuffersSwap();
     mav_windowSet(mav_win_left);
   }
@@ -275,10 +335,37 @@ void mavlib_TRfn3(void)
   if (mav_opt_output==MAV_VERBOSE && os!=0) fprintf(stderr, "using %i oversamples\n", os);
   
   if (os!=0) {
+
+    /* Display image and prompt for antialiased or oversized image */
+    mav_windowSet(mav_win_left);
+    mav_gfxListExec(DL);
+    mav_stringDisplay(mav_win_left, "Anti-aliased (a) or oversized (o) image?", MAV_COLOUR_BLACK, 0, -0.9, 0.9);
+    mav_gfxWindowBuffersSwap();
+
+    if (mav_opt_stereo) {
+      mav_windowSet(mav_win_right);
+      mav_gfxListExec(DR);
+      mav_stringDisplay(mav_win_left, "Anti-aliased (a) or oversized (o) image?", MAV_COLOUR_BLACK, 0, -0.9, 0.9);
+      mav_gfxWindowBuffersSwap();
+      mav_windowSet(mav_win_left);
+    }
+
+    /* Get response from user */
+    mav_callbackKeyboardSet(mav_win_all, mav_class_world, mavlib_TRKey2);
+    mavlib_TROS=-1;
+    rv= -1;
+    mav_timerStart(&tim);
+    do {
+      rv= mav_eventsCheck();
+      mav_timerStop(&tim);
+    } while (!(tim.wall>5 || rv==-862));
+    mav_callbackKeyboardSet(mav_win_all, mav_class_world, (MAV_callbackKeyboardFn) keyb);
+    if (mavlib_TROS==-1) aa= 1;
+
     /* Set line width and dump windows */
     lw= mav_gfxLineWidthGet();
     mav_gfxLineWidthSet(lw*os);
-    mavlib_TRDump(mav_win_left, os, DL);
+    mavlib_TRDump(mav_win_left, os, DL, aa);
     mav_gfxLineWidthSet(lw);
 
     if (mav_opt_stereo) {
@@ -287,7 +374,7 @@ void mavlib_TRfn3(void)
       mav_windowSet(mav_win_right);
       lw= mav_gfxLineWidthGet();
       mav_gfxLineWidthSet(lw*os);
-      mavlib_TRDump(mav_win_right, os, DR);
+      mavlib_TRDump(mav_win_right, os, DR, aa);
       mav_gfxLineWidthSet(lw);
     }
   }
