@@ -115,7 +115,19 @@ float mavlib_vradReadFloat(FILE *f)
 
   memcpy(&v, buf2, 4);
 #else
+#ifdef WIN32
+  char buf[4], buf2[4];
+  fread(&buf, 1, 4, f);
+  
+  buf2[0]= buf[3];
+  buf2[1]= buf[2];
+  buf2[2]= buf[1];
+  buf2[3]= buf[0];
+
+  memcpy(&v, buf2, 4);
+#else
   fread(&v, sizeof(float), 1, f);
+#endif
 #endif
   
   return v;
@@ -145,6 +157,8 @@ int mav_vradRead(char *filename, MAV_vrad *vrad, MAV_matrix m)
 
     elem->type= mavlib_vradReadInt(file);
 
+    if (feof(file)) break;
+
     /* Number of vertices depends on type */
     switch (elem->type) {
       case 0 :
