@@ -35,6 +35,9 @@
 #include <gl/gl.h>
 float mavlib_bk[4];
 #else
+#ifdef WIN32
+#include <windows.h>
+#endif
 #include "GL/glu.h"
 #ifdef GL_EXT_texture_object
 GLuint *mavlib_bindTextureIndex;
@@ -524,8 +527,20 @@ void mav_gfxLightPos(MAV_light info)
     pv[1]=info.pos.y;
     pv[2]=info.pos.z;
     pv[3]=1.0;
-
-    glLightfv(whichlight, GL_POSITION, pv);
+    
+    if (info.positioning==MAV_LIGHT_RELATIVE) 
+    {
+      /* Position is relative to eye */
+      glPushMatrix(); 
+      glLoadIdentity();
+      glLightfv(whichlight, GL_POSITION, pv);
+      glPopMatrix();
+    }
+    else
+    {
+      /* Position is in world coords */
+      glLightfv(whichlight, GL_POSITION, pv);
+    }
   }
 #endif
 #endif
@@ -879,10 +894,14 @@ void mav_gfxTextureUse(MAV_texture tex, MAV_texEnvFn pTexEnv)
 #endif
 #endif
   {
+#ifdef WIN32
+    glTexImage2D(GL_TEXTURE_2D, 0, 4, tex.width, tex.height, 0, GL_RGBA, GL_UNSIGNED_BYTE, tex.mem);
+#else
 #ifdef GL_EXT_abgr
     glTexImage2D(GL_TEXTURE_2D, 0, 4, tex.width, tex.height, 0, GL_ABGR_EXT, GL_UNSIGNED_BYTE, tex.mem);
 #else
     fprintf(stderr, "Warning: no ABGR texture extension, ignoring.\n");
+#endif
 #endif
   }
 
@@ -1100,7 +1119,14 @@ void mav_gfxDepthMaskSet(int i)
     zwritemask(0);
   }
 #else
-  glDepthMask(i);
+  if (i)
+  {
+    glDepthMask(1);
+  }
+  else
+  {
+    glDepthMask(0);
+  }
 #endif
 #endif
 }
