@@ -27,16 +27,30 @@
 
 
 #include "mavlib_kernel.h"
-#include <strings.h>
+#include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#ifndef WIN32
 #include <unistd.h>
 #ifdef MAV_LINUX
 int gethostname(char *name, size_t len);
 #endif
 #include <dlfcn.h>
+#endif
+
+#ifdef WIN32
+int mav_gfxModuleInit(void); /* WIN32 */
+int mav_callbacksModuleInit(void);
+int mav_SMSModuleInit(void);
+int mav_windowsModuleInit(void);
+int mav_navigationModuleInit(void);
+int mav_objectsModuleInit(void);
+#endif
 
 const MAV_vector MAV_NULL_VECTOR={0,0,0};
+const MAV_vector MAV_X_VECTOR={1,0,0};
+const MAV_vector MAV_Y_VECTOR={0,1,0};
+const MAV_vector MAV_Z_VECTOR={0,0,1};
 const MAV_matrix MAV_ID_MATRIX={{{1,0,0,0},{0,1,0,0},{0,0,1,0},{0,0,0,1}}};
 const MAV_quaternion MAV_ID_QUATERNION={1,0,0,0};
 
@@ -48,6 +62,7 @@ MAV_SMSCallback *mav_SMSCallback_delete;
 MAV_SMSCallback *mav_SMSCallback_objectRmv;
 MAV_list *mav_module_list;
 MAV_list *mav_sms_list;
+MAV_list *mav_palette_list;
 MAV_viewParams mav_vp_default;
 int mavlib_usedWin[MAV_MAX_WIN];
 float mav_fps;
@@ -69,9 +84,10 @@ void *mavlib_dlh= NULL;
 
 char *mav_kernelID(void)
 {
-  return "GNU Maverik v5.0";
+  return "GNU Maverik v5.1";
 }
 
+#ifndef WIN32
 void mavlib_execInit(char *buf)
 {
   MAV_moduleInitFn fn;
@@ -87,11 +103,15 @@ void mavlib_execInit(char *buf)
     exit(1);
   }
 }
+#endif
 
 void mav_initialise(void)
 {
   FILE *f;
-  char buf[100], homepath[200];
+#ifndef WIN32
+  char buf[100];
+#endif
+  char homepath[200];
   int i, defaultInit=0;
   char *defaultMod[]={"mav_gfxModuleInit","mav_callbacksModuleInit","mav_SMSModuleInit","mav_windowsModuleInit","mav_navigationModuleInit","mav_objectsModuleInit",0};
 
@@ -110,14 +130,15 @@ void mav_initialise(void)
   /* create the global class handle */
   mav_class_all= mav_classNew();
 
+  /* create a list of all windows, SMS's and palette */
+  mav_win_list= mav_listNew();
+  mav_sms_list= mav_listNew();
+  mav_palette_list= mav_listNew();
+
   /* create the globla palette handle */
   mav_palette_default= mav_paletteNew();
   mav_windowPaletteSet(mav_win_all, mav_palette_default);
 
-  /* create a list of all windows and SMS's */
-  mav_win_list= mav_listNew();
-  mav_sms_list= mav_listNew();
-
   /* create a list frame functions */
   mavlib_frame0_list= mav_listNew();
   mavlib_frame1_list= mav_listNew();
@@ -162,7 +183,11 @@ void mav_initialise(void)
   }
 
   /* get hostname */
+#ifdef WIN32
+  strcpy(mav_hostName, "hostname"); /* WIN32 */
+#else
   gethostname(mav_hostName, 200);
+#endif
 
   /* multisample hint  */
   if (mav_opt_multiSample==MAV_DONTCARE) {
@@ -192,6 +217,7 @@ void mav_initialise(void)
   }
 
   /* initalise each module defined */
+#ifndef WIN32
 #ifdef MAV_SUNOS4
   mavlib_dlh= dlopen(0, 1);
 #else
@@ -219,6 +245,14 @@ void mav_initialise(void)
     
     fclose(f);
   }
+#else
+  mav_gfxModuleInit(); /* WIN32 */
+  mav_callbacksModuleInit();
+  mav_SMSModuleInit();
+  mav_windowsModuleInit();
+  mav_navigationModuleInit();
+  mav_objectsModuleInit();
+#endif
 }
 
 
@@ -267,6 +301,10 @@ long sginap(long ticks);
 void usleep(unsigned long usec);
 #endif
 
+#ifdef WIN32
+#include <time.h>
+#endif
+
 void mav_sleep(float len)
 {
 #ifdef MAV_IRIX5
@@ -274,10 +312,15 @@ void mav_sleep(float len)
 #else
 #ifdef MAV_IRIX6
   sginap((long) (len*CLK_TCK));
+#else
+#ifdef WIN32
+  clock_t goal= clock()+(len*CLOCKS_PER_SEC);
+  while (goal>clock());
 #else
   usleep(len*1.0E+6);
 #endif
 #endif
+#endif
 }
 
 
