@@ -31,8 +31,10 @@
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
+#ifndef WIN32
 #include <sys/types.h>
 #include <unistd.h>
+#endif
 char *strdup (const char *s1);
 
 MAV_surfaceParams mavlib_surfaceParams={-1,-1,-1,-1};
@@ -52,6 +54,10 @@ int mav_opt_maxTextures=150;
 int mav_opt_maxFonts=10;
 int mav_opt_maxLights=5;
 
+extern int mavlib_voodoo;
+
+
+
 /* Routine to enable or disable surface params */
 
 void mav_surfaceParamsFlagSet(int mode)
@@ -173,6 +179,9 @@ MAV_palette *mav_paletteNew(void)
   MAV_palette *rv= (MAV_palette *) mav_malloc(sizeof(MAV_palette));
   int i;
 
+  /* Add to list of palettes */
+  mav_listItemAdd(mav_palette_list, (void *) rv);
+
  /* 
      Clear colour, material, texture and light defined lists 
      Give each a unique ID to define and bind against.
@@ -205,6 +214,7 @@ MAV_palette *mav_paletteNew(void)
     rv->lightlist[i].id= (mavlib_num_palettes*mav_opt_maxLights)+i;
     rv->lightlist[i].index=i;
     rv->lightlist[i].defined=0;
+    rv->lightlist[i].positioning= MAV_LIGHT_RELATIVE;
   }
 
   rv->fontlist= mav_malloc(mav_opt_maxFonts*sizeof(MAV_font));
@@ -287,6 +297,25 @@ void mavlib_lightingModelUpd(MAV_palette *p)
 
 
 
+/* Routine to handle absolute lights */
+
+void mavlib_lightPosFix(void)
+{
+  MAV_palette *p;
+  int i;
+
+  mav_listPointerReset(mav_palette_list);
+  
+  while (mav_listItemNext(mav_palette_list, (void **) &p)) {
+    for (i=0; i<mav_opt_maxLights; i++) {
+      if (p->lightlist[i].defined && p->lightlist[i].positioning==MAV_LIGHT_ABSOLUTE) mavlib_lightPosUpd(i,p);
+    }
+  }
+}
+
+
+
+
 /* Routine to set the palette for a given window */
 
 void mav_windowPaletteSet(MAV_window *w, MAV_palette *p)
@@ -456,6 +485,18 @@ int mavlib_readPPM(char *filename, int *width, int *height, unsigned long **mem)
   /* if binary, read since white space char */
   if (bin) fscanf(file, "%c", &str[0]);
 
+  /* re-open as binary file, but maintaining position, if appropriate */
+  if (bin) {
+    long offset= ftell(file);
+    fclose(file);
+    file= fopen(filename, "rb");
+    if (!file) {
+      if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Error: can not re-open as binary file, ignoring\n");
+      return(MAV_FALSE);
+    }
+    fseek(file, offset, SEEK_SET);
+  }
+
 /* Parse */
   
   *mem=(unsigned long *) mav_malloc(*width**height*sizeof(unsigned long));
@@ -466,10 +507,10 @@ int mavlib_readPPM(char *filename, int *width, int *height, unsigned long **mem)
 
       if (bin) 
       {
-	fscanf(file, "%c%c%c", &str[0], &str[1], &str[2]);
-	r= str[0];
-	g= str[1];
-	b= str[2];
+	fread(str, 1, 3, file);
+	r= (unsigned char) str[0];
+	g= (unsigned char) str[1];
+	b= (unsigned char) str[2];
       }
       else
       {
@@ -522,8 +563,9 @@ int mav_paletteTextureSet(MAV_palette *p, int index, char *filename)
     {
       rv= mavlib_readPPM(filename, &p->texlist[index].width, &p->texlist[index].height, &p->texlist[index].mem);
     }
-    else 
+    else
     {
+#ifndef WIN32
       char buf[500], f1[500], f2[500];
       int srv;
 
@@ -542,22 +584,63 @@ int mav_paletteTextureSet(MAV_palette *p, int index, char *filename)
 	}
       }
       else
+#endif
       {
 	if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Error: can not read texture file %s, ignoring\n", filename);	
 	rv= MAV_FALSE;
       }
     }
     
-    /* Only interger powers of 2 allowed for height and width */
+    /* Only interger powers of 2 allowed for height and width, use ImageMagick's covert program to scale */
     
-    if (rv) {      
+    if (rv) {
       width= p->texlist[index].width;
       height= p->texlist[index].height;
       
       if (fabs(log10(width)/L2 - (int) (log10(width)/L2))>0.0001 || 
 	  fabs(log10(height)/L2 - (int) (log10(height)/L2))>0.0001) {
-	if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Error: width and height (%ix%i) must be an integer power of 2, ignoring\n", width, height);
-	rv=MAV_FALSE;
+	int tw, th;
+
+	/* Calculate required size */
+	tw= pow(2, ((int) (log10(width)/L2))+1);
+	th= pow(2, ((int) (log10(height)/L2))+1);
+	
+	/* Max texture size of 256x256 on Voodoo */
+	if (mavlib_voodoo) {
+	  if (tw>256) tw=256;
+	  if (th>256) th=256;
+	}
+
+	/* Free old texture memory */
+	mav_free(p->texlist[index].mem);
+
+	{
+#ifndef WIN32
+	  char buf[500], f1[500], f2[500];
+	  int srv;
+
+	  sprintf(f1, "/tmp/mavtex%i.ppm", getpid());
+	  sprintf(f2, "/tmp/mavtexlog%i", getpid());
+
+	  sprintf(buf, "convert -geometry %ix%i! %s %s 2>%s", tw, th, filename, f1, f2);
+
+	  srv= system(buf);
+
+	  if (srv==0)
+	  {
+	    rv= mavlib_readPPM(f1, &p->texlist[index].width, &p->texlist[index].height, &p->texlist[index].mem);
+	    if (rv==MAV_TRUE) {
+	      sprintf(buf, "rm -f %s %s", f1, f2);
+	      system(buf);
+	    }
+	  }
+	  else
+#endif
+	  {
+	    if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Error: width and height (%ix%i) must be an integer power of 2, ignoring\n", width, height);
+	    rv=MAV_FALSE;
+	  }
+	}
       }
     }
     
@@ -820,7 +903,7 @@ void mav_paletteLightSet(MAV_palette *p, int index, float a1, float a2, float a3
 
 
 
-/* Routine to position a light. N.B. pos is multipled by current view matrix */
+/* Routine to position a light */
 
 void mav_paletteLightPos(MAV_palette *p, int index, MAV_vector pos) 
 {
@@ -832,11 +915,11 @@ void mav_paletteLightPos(MAV_palette *p, int index, MAV_vector pos)
   }
   else
   {
-    /* Warn if index is not already defined */
+    /* Warn if index is not defined */
     
-    if (!p->lightlist[index].defined && mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: Light index %i not defined in palette, overwriting\n", index);
+    if (!p->lightlist[index].defined && mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: Light index %i not defined in palette\n", index);
     
-    p->lightlist[index].pos= pos;    
+    p->lightlist[index].pos= pos;
 
     /* set changes in all windows with this palette */
     mavlib_lightPosUpd(index, p);
@@ -845,6 +928,36 @@ void mav_paletteLightPos(MAV_palette *p, int index, MAV_vector pos)
 
 
 
+/* Routine to set positioning (absolute or relative) of a light */
+
+void mav_paletteLightPositioning(MAV_palette *p, int index, int pos) 
+{
+  /* Check index is within range */
+  
+  if (index >= mav_opt_maxLights) 
+  {
+    if (mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Error: Light index %i too big (max %i), ignoring\n", index, mav_opt_maxLights-1);
+  }
+  else
+  {
+    /* Warn if index is not defined */
+    
+    if (!p->lightlist[index].defined && mav_opt_output==MAV_VERBOSE) fprintf(stderr, "Warning: Light index %i not defined in palette\n", index);
+
+    /* set positioning */
+    if (pos==MAV_LIGHT_RELATIVE)
+    {
+      p->lightlist[index].positioning= MAV_LIGHT_RELATIVE;
+    }
+    else
+    {
+      p->lightlist[index].positioning= MAV_LIGHT_ABSOLUTE;
+    }
+  }
+}
+
+
+
 /* Routine to define the lighting model for a given palette */
 
 void mav_paletteLightingModelSet(MAV_palette *p, float a1, float a2, float a3, float a4, int local_viewer)
