@@ -26,6 +26,12 @@
 */
 
 
+#ifdef WIN32
+#pragma warning( disable : 4244 ) /* '=' : conversion from 'const double ' to 'float ', possible loss of data */
+#pragma warning( disable : 4018 ) /* '<' : signed/unsigned mismatch */
+#pragma warning( disable : 4305 ) /* '=' : truncation from 'const double ' to 'float ' */
+#endif
+
 #include "mav_kernel.h"
 #include "mav_gfx.h"
 
@@ -47,6 +53,7 @@ extern MAV_list *mavlib_deviceEvent_list;
 void mavlib_setUpObjectTables(void); 
 void mavlib_objectTablesAddSMS(MAV_object *, MAV_SMS *);
 void mavlib_objectTablesRemoveSMS(MAV_object *, MAV_SMS *);
+void mavlib_lightPosFix(void);
 
 typedef struct {
   MAV_object *the_obj;
