@@ -121,7 +121,7 @@ int mavlib_ac3d_findMatPlace(float *dif, float *amb, float *emis, float *spec, f
   }
   
   /* cant find empty slot */
-  mavlib_ac3d_fprintf("Error: maximum number of materials exceeded.\n");
+  if (mav_opt_output==MAV_VERBOSE) mavlib_ac3d_fprintf("Error: maximum number of materials exceeded.\n");
   return -1;
 }
 
@@ -154,7 +154,7 @@ int mavlib_ac3d_findColPlace(float *amb, float trans)
   }
 
   /* cant find empty slot */
-  mavlib_ac3d_fprintf("Error: maximum number of colours exceeded.\n");
+  if (mav_opt_output==MAV_VERBOSE) mavlib_ac3d_fprintf("Error: maximum number of colours exceeded.\n");
   return -1;
 }
 
@@ -162,10 +162,10 @@ int mavlib_ac3d_findColPlace(float *amb, float trans)
 
 /* Routine to return an empty or matching texture index */
 
-int mavlib_ac3d_texLookUp(char *nm)
+int mavlib_ac3d_texLookUp2(char *nm)
 {
   int i;
-  
+
   /* check if texture has already been defined */
   for (i=0; i<mav_opt_maxTextures; i++) {
     if (mav_palette_default->texlist[i].defined) {
@@ -182,8 +182,70 @@ int mavlib_ac3d_texLookUp(char *nm)
   }
   
   /* cant find empty slot */
-  mavlib_ac3d_fprintf("Error: maximum number of textures exceeded.\n");
-  return -1;
+  return -2;
+}
+
+char *mavlib_ac3d_origFile;
+
+int mavlib_ac3d_texLookUp(char *nm)
+{
+  char path[500], fn2[500];
+  int i, optbk, rv;
+
+  /* Silent output */
+  optbk= mav_opt_output;
+  mav_opt_output= MAV_SILENT;
+
+  if (strstr(nm, "/"))
+  { 
+    /* Texture filename contains path */
+    rv= mavlib_ac3d_texLookUp2(nm);
+  }
+  else
+  {
+    /* Texture filename is relative */
+
+    /* Find path of AC3D model filename */
+    strcpy(path, mavlib_ac3d_origFile);
+    for (i=strlen(path)-1; i>=0 && path[i]!='/'; i--);
+    if (i==-1) 
+    {
+      path[0]='.';
+      path[1]=0;
+    }
+    else if (i==0) 
+    {
+      path[0]='/';
+      path[1]=0;
+    }
+    else
+    {
+      path[i]=0;
+    }
+    
+    /* Account for AC3D model path when looking for texture */
+    sprintf(fn2, "%s/%s", path, nm);
+    rv= mavlib_ac3d_texLookUp2(fn2);
+
+    if (rv==-1) {
+      /* Failed, so try looking in textures sub-directory */
+      sprintf(fn2, "%s/textures/%s", path, nm);
+      rv= mavlib_ac3d_texLookUp2(fn2);
+    }
+  }
+
+  /* Restore output */
+  mav_opt_output= optbk;
+
+  /* Print error messages as appropriate */
+  if (rv==-1 && mav_opt_output==MAV_VERBOSE) {
+    sprintf(path, "Warning: failed to read texture %s, ignoring.\n", nm);
+    mavlib_ac3d_fprintf(path);
+  }
+
+  if (rv==-2 && mav_opt_output==MAV_VERBOSE) mavlib_ac3d_fprintf("Error: maximum number of textures exceeded.\n");
+
+  return rv;
 }
 
 
@@ -577,7 +639,7 @@ int mavlib_ac3d_parseObject(MAV_facet *fc, MAV_polyline *pl, MAV_matrix mat)
 	    if (obj->texture[0]) 
 	    {
 	      fc->sp[fc->npolys]= mav_surfaceParamsNew(MAV_LIT_TEXTURE, 0, mavlib_ac3d_matLookUp[obj->matindex[i]], mavlib_ac3d_texLookUp(obj->texture));
-	      if (fc->sp[fc->npolys]->texture==-1) return -1;
+	      if (fc->sp[fc->npolys]->texture<0) return -1;
 	    }
 	    else
 	    {
@@ -669,6 +731,7 @@ int mavlib_compositeAC3DRead(char *filename, MAV_composite *c, MAV_matrix m)
   char str[100];
   int ns, sp, nolines=0, nosurf=0, otbk;
 
+  mavlib_ac3d_origFile= filename;
   mavlib_ac3d_nl=0;
 
   /* open file */
