@@ -469,7 +469,11 @@ int mavlib_SolveCubic(double c[4], double s[3])
         }
         else /* one single and one double solution */
         {
+#ifdef WIN32
+            double u = pow(-q, 1.0/3.0);
+#else
             double u = cbrt(-q);
+#endif
             s[ 0 ] = 2 * u;
             s[ 1 ] = - u;
             num = 2;
@@ -488,9 +492,13 @@ int mavlib_SolveCubic(double c[4], double s[3])
     else /* one real solution */
     {
         double sqrt_D = sqrt(D);
+#ifdef WIN32
+        double u = pow(sqrt_D - q, 1.0/3.0);
+        double v = - pow(sqrt_D + q, 1.0/3.0);
+#else
         double u = cbrt(sqrt_D - q);
         double v = - cbrt(sqrt_D + q);
-
+#endif
         s[ 0 ] = u + v;
         num = 1;
     }
