@@ -142,7 +142,7 @@ int mavlib_dealWithMouseEvent(int *info)
   }
 
   for (i=0; i<MAV_MODIFIER_MAX; i++) {
-    if (info[i+7]) 
+    if (info[i+7])  /* This is inconsistent with 0=pressed, 1=released ! */
     {
       me.modifiers[i]= MAV_PRESSED;
     }
@@ -271,13 +271,23 @@ void mav_mouseDraw(void)
     mav_surfaceParamsUse(mavlib_mouseSp);
 
     mav_gfxLineBegin();
+#ifdef WIN32
+    mav_gfxVertex(mav_vectorAdd(mav_mouse_pos, mav_vectorScalar(mav_win_current->vp->trans_right, -100.0)));
+    mav_gfxVertex(mav_vectorAdd(mav_mouse_pos, mav_vectorScalar(mav_win_current->vp->trans_right, +100.0)));
+#else
     mav_gfxVertex(mav_vectorAdd(mav_mouse_pos, mav_vectorScalar(mav_win_current->vp->trans_right, -10000.0)));
     mav_gfxVertex(mav_vectorAdd(mav_mouse_pos, mav_vectorScalar(mav_win_current->vp->trans_right, +10000.0)));
+#endif
     mav_gfxLineEnd();
     
     mav_gfxLineBegin();
+#ifdef WIN32
+    mav_gfxVertex(mav_vectorAdd(mav_mouse_pos, mav_vectorScalar(mav_win_current->vp->trans_up, -100.0)));
+    mav_gfxVertex(mav_vectorAdd(mav_mouse_pos, mav_vectorScalar(mav_win_current->vp->trans_up, +100.0)));
+#else
     mav_gfxVertex(mav_vectorAdd(mav_mouse_pos, mav_vectorScalar(mav_win_current->vp->trans_up, -10000.0)));
     mav_gfxVertex(mav_vectorAdd(mav_mouse_pos, mav_vectorScalar(mav_win_current->vp->trans_up, +10000.0)));
+#endif
     mav_gfxLineEnd();
     
     if (mav_win_current!=orig_win) mav_windowSet(orig_win);
