@@ -30,10 +30,14 @@
 #include <stdio.h>
 #include <math.h>
 #include <stdlib.h>
+#ifndef WIN32
 #include <dlfcn.h>
+#endif
 #include <string.h>
+#ifndef WIN32
 #include <sys/types.h>
 #include <unistd.h>
+#endif
 
 int mavlib_defaultColours[][3]={
 {148,   0, 211},             /* darkviolet */
@@ -259,6 +263,9 @@ void mavlib_cf1(void)
 
 void mavlib_cf12(void)
 {
+#ifdef WIN32
+  fprintf(stderr, "Warning: dynamic loading of modules is not supported under Windows, ignoring\n"); /* WIN32 */
+#else
   char buf[500], filename[500], initName[100];
   void *dlh;
   MAV_moduleInitFn fn;
@@ -329,6 +336,7 @@ void mavlib_cf12(void)
       return;
     }
   }
+#endif
 }
 
 
@@ -342,8 +350,11 @@ char *mav_windowsModuleID(void)
 
 int mav_windowsModuleInit(void)
 {
+#ifndef WIN32
   FILE *f;
-  char buf[500], sbuf[510], filename[500];
+  char filename[500];
+#endif
+  char buf[500], sbuf[510];
   int lx, ly, lw=0, lh=0, rx, ry, rw, rh, i;
 
   /* add the new module */  
@@ -538,6 +549,9 @@ int mav_windowsModuleInit(void)
     }
 
     /* find name of executable */
+#ifdef WIN32
+    strcpy(buf, "Maverik"); /* WIN32 */
+#else
     sprintf(filename, "/tmp/mavname%i", getpid());
 #ifdef MAV_LINUX
     sprintf(buf, "ps a | awk '{if ($1==%i) print $5}' 2>&1 >%s", getpid(), filename);
@@ -557,6 +571,7 @@ int mav_windowsModuleInit(void)
     }
     sprintf(sbuf, "rm -f %s", filename);
     system(sbuf);
+#endif
 
     /* open the windows with the correct name */
     if (mav_opt_stereo) 
